@@ -8,7 +8,8 @@ from datetime import timedelta
 
 import timezone_field
 from celery import current_app, schedules
-from cron_descriptor import get_description
+from cron_descriptor import (FormatException, MissingFieldException,
+                             WrongArgumentException, get_description)
 from django.conf import settings
 from django.core.exceptions import MultipleObjectsReturned, ValidationError
 from django.core.validators import MaxValueValidator, MinValueValidator
@@ -64,7 +65,7 @@ def crontab_schedule_celery_timezone():
     """Return timezone string from Django settings ``CELERY_TIMEZONE`` variable.
 
     If is not defined or is not a valid timezone, return ``"UTC"`` instead.
-    """  # noqa: E501
+    """
     try:
         CELERY_TIMEZONE = getattr(
             settings, '%s_TIMEZONE' % current_app.namespace)
@@ -273,14 +274,6 @@ class CrontabSchedule(models.Model):
             'Cron Hours to Run. Use "*" for "all". (Example: "8,20")'),
         validators=[validators.hour_validator],
     )
-    day_of_week = models.CharField(
-        max_length=64, default='*',
-        verbose_name=_('Day(s) Of The Week'),
-        help_text=_(
-            'Cron Days Of The Week to Run. Use "*" for "all", Sunday '
-            'is 0 or 7, Monday is 1. (Example: "0,5")'),
-        validators=[validators.day_of_week_validator],
-    )
     day_of_month = models.CharField(
         max_length=31 * 4, default='*',
         verbose_name=_('Day(s) Of The Month'),
@@ -297,6 +290,14 @@ class CrontabSchedule(models.Model):
             '(Example: "1,12")'),
         validators=[validators.month_of_year_validator],
     )
+    day_of_week = models.CharField(
+        max_length=64, default='*',
+        verbose_name=_('Day(s) Of The Week'),
+        help_text=_(
+            'Cron Days Of The Week to Run. Use "*" for "all", Sunday '
+            'is 0 or 7, Monday is 1. (Example: "0,5")'),
+        validators=[validators.day_of_week_validator],
+    )
 
     timezone = timezone_field.TimeZoneField(
         default=crontab_schedule_celery_timezone,
@@ -316,11 +317,34 @@ class CrontabSchedule(models.Model):
 
     @property
     def human_readable(self):
-        human_readable = get_description('{} {} {} {} {}'.format(
+        try:
+            c = schedules.crontab(
+                minute=self.minute,
+                hour=self.hour,
+                day_of_week=self.day_of_week,
+                day_of_month=self.day_of_month,
+                month_of_year=self.month_of_year,
+            )
+            if c.day_of_week and set(c.day_of_week) == set(range(7)):
+                day_of_week = "*"
+            else:
+                day_of_week = cronexp(",".join(map(str, c.day_of_week)))
+        except ValueError:
+            day_of_week = cronexp(self.day_of_week)
+
+        cron_expression = '{} {} {} {} {}'.format(
             cronexp(self.minute), cronexp(self.hour),
             cronexp(self.day_of_month), cronexp(self.month_of_year),
-            cronexp(self.day_of_week)
-        ))
+            day_of_week
+        )
+        try:
+            human_readable = get_description(cron_expression)
+        except (
+            MissingFieldException,
+            FormatException,
+            WrongArgumentException
+        ):
+            return f'{cron_expression} {str(self.timezone)}'
         return f'{human_readable} {str(self.timezone)}'
 
     def __str__(self):
@@ -366,6 +390,11 @@ class CrontabSchedule(models.Model):
         except MultipleObjectsReturned:
             return cls.objects.filter(**spec).first()
 
+    def due_start_time(self, initial_start_time, tz):
+        start_time = initial_start_time.astimezone(tz)
+        start, ends_in, now = self.schedule.remaining_delta(start_time)
+        return start + ends_in
+
 
 class PeriodicTasks(models.Model):
     """Helper table for tracking updates to periodic tasks.
@@ -379,6 +408,10 @@ class PeriodicTasks(models.Model):
     ident = models.SmallIntegerField(default=1, primary_key=True, unique=True)
     last_update = models.DateTimeField(null=False)
 
+    class Meta:
+        verbose_name = _('periodic task track')
+        verbose_name_plural = _('periodic task tracks')
+
     @classmethod
     def changed(cls, instance, **kwargs):
         if not instance.no_changes:
@@ -635,3 +668,9 @@ class PeriodicTask(models.Model):
     @property
     def schedule(self):
         return self.scheduler.schedule
+
+    def due_start_time(self, tz):
+        if self.crontab:
+            return self.crontab.due_start_time(self.start_time, tz)
+        else:
+            return self.start_time
