@@ -4,6 +4,11 @@ import logging
 import math
 from multiprocessing.util import Finalize
 
+try:
+    from zoneinfo import ZoneInfo  # Python 3.9+
+except ImportError:
+    from backports.zoneinfo import ZoneInfo  # Python 3.8
+
 from celery import current_app, schedules
 from celery.beat import ScheduleEntry, Scheduler
 from celery.utils.log import get_logger
@@ -11,6 +16,8 @@ from celery.utils.time import maybe_make_aware
 from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
 from django.db import close_old_connections, transaction
+from django.db.models import Case, F, IntegerField, Q, When
+from django.db.models.functions import Cast
 from django.db.utils import DatabaseError, InterfaceError
 from kombu.utils.encoding import safe_repr, safe_str
 from kombu.utils.json import dumps, loads
@@ -18,12 +25,13 @@ from kombu.utils.json import dumps, loads
 from .clockedschedule import clocked
 from .models import (ClockedSchedule, CrontabSchedule, IntervalSchedule,
                      PeriodicTask, PeriodicTasks, SolarSchedule)
-from .utils import NEVER_CHECK_TIMEOUT
+from .utils import NEVER_CHECK_TIMEOUT, aware_now, now
 
 # This scheduler must wake up more frequently than the
 # regular of 5 minutes because it needs to take external
 # changes to the schedule into account.
 DEFAULT_MAX_INTERVAL = 5  # seconds
+SCHEDULE_SYNC_MAX_INTERVAL = 300  # 5 minutes
 
 ADD_ENTRY_ERROR = """\
 Cannot add entry %r to database schedule: %r. Contents: %r
@@ -77,14 +85,15 @@ class ModelEntry(ScheduleEntry):
         if getattr(model, 'expires_', None):
             self.options['expires'] = getattr(model, 'expires_')
 
-        self.options['headers'] = loads(model.headers or '{}')
-        self.options['periodic_task_name'] = model.name
+        headers = loads(model.headers or '{}')
+        headers['periodic_task_name'] = model.name
+        self.options['headers'] = headers
 
         self.total_run_count = model.total_run_count
         self.model = model
 
         if not model.last_run_at:
-            model.last_run_at = self._default_now()
+            model.last_run_at = model.date_changed or self._default_now()
             # if last_run_at is not set and
             # model.start_time last_run_at should be in way past.
             # This will trigger the job to run at start_time
@@ -110,15 +119,24 @@ class ModelEntry(ScheduleEntry):
             now = self._default_now()
             if getattr(settings, 'DJANGO_CELERY_BEAT_TZ_AWARE', True):
                 now = maybe_make_aware(self._default_now())
-
             if now < self.model.start_time:
                 # The datetime is before the start date - don't run.
                 # send a delay to retry on start_time
-                delay = math.ceil(
-                    (self.model.start_time - now).total_seconds()
-                )
+                current_tz = now.tzinfo
+                start_time = self.model.due_start_time(current_tz)
+                time_remaining = start_time - now
+                delay = math.ceil(time_remaining.total_seconds())
+
                 return schedules.schedstate(False, delay)
 
+        # EXPIRED TASK: Disable task when expired
+        if self.model.expires is not None:
+            now = self._default_now()
+            if now >= self.model.expires:
+                self._disable(self.model)
+                # Don't recheck
+                return schedules.schedstate(False, NEVER_CHECK_TIMEOUT)
+
         # ONE OFF TASK: Disable one off tasks after they've ran once
         if self.model.one_off and self.model.enabled \
                 and self.model.total_run_count > 0:
@@ -226,6 +244,7 @@ class DatabaseScheduler(Scheduler):
     _last_timestamp = None
     _initial_read = True
     _heap_invalidated = False
+    _last_full_sync = None
 
     def __init__(self, *args, **kwargs):
         """Initialize the database scheduler."""
@@ -244,13 +263,144 @@ class DatabaseScheduler(Scheduler):
     def all_as_schedule(self):
         debug('DatabaseScheduler: Fetching database schedule')
         s = {}
-        for model in self.Model.objects.enabled():
+        for model in self.enabled_models():
             try:
                 s[model.name] = self.Entry(model, app=self.app)
             except ValueError:
                 pass
         return s
 
+    def enabled_models(self):
+        """Return list of enabled periodic tasks.
+
+        Allows overriding how the list of periodic tasks is fetched without
+        duplicating the filtering/querying logic.
+        """
+        return list(self.enabled_models_qs())
+
+    def enabled_models_qs(self):
+        next_schedule_sync = now() + datetime.timedelta(
+            seconds=SCHEDULE_SYNC_MAX_INTERVAL
+        )
+        exclude_clock_tasks_query = Q(
+            clocked__isnull=False,
+            clocked__clocked_time__gt=next_schedule_sync
+        )
+
+        exclude_cron_tasks_query = self._get_crontab_exclude_query()
+
+        # Combine the queries for optimal database filtering
+        exclude_query = exclude_clock_tasks_query | exclude_cron_tasks_query
+
+        # Fetch only the tasks we need to consider
+        return self.Model.objects.enabled().exclude(exclude_query)
+
+    def _get_crontab_exclude_query(self):
+        """
+        Build a query to exclude crontab tasks based on their hour value,
+        adjusted for timezone differences relative to the server.
+
+        This creates an annotation for each crontab task that represents the
+        server-equivalent hour, then filters on that annotation.
+        """
+        # Get server time based on Django settings
+
+        server_time = aware_now()
+        server_hour = server_time.hour
+
+        # Window of +/- 2 hours around the current hour in server tz.
+        hours_to_include = [
+            (server_hour + offset) % 24 for offset in range(-2, 3)
+        ]
+        hours_to_include += [4]  # celery's default cleanup task
+
+        # Regex pattern to match only numbers
+        # This ensures we only process numeric hour values
+        numeric_hour_pattern = r'^\d+$'
+
+        # Get all tasks with a simple numeric hour value
+        numeric_hour_tasks = CrontabSchedule.objects.filter(
+            hour__regex=numeric_hour_pattern
+        )
+
+        # Annotate these tasks with their server-hour equivalent
+        annotated_tasks = numeric_hour_tasks.annotate(
+            # Cast hour string to integer
+            hour_int=Cast('hour', IntegerField()),
+
+            # Calculate server-hour based on timezone offset
+            server_hour=Case(
+                # Handle each timezone specifically
+                *[
+                    When(
+                        timezone=timezone_name,
+                        then=(
+                            F('hour_int')
+                            + self._get_timezone_offset(timezone_name)
+                            + 24
+                        ) % 24
+                    )
+                    for timezone_name in self._get_unique_timezone_names()
+                ],
+                # Default case - use hour as is
+                default=F('hour_int')
+            )
+        )
+
+        excluded_hour_task_ids = annotated_tasks.exclude(
+            server_hour__in=hours_to_include
+        ).values_list('id', flat=True)
+
+        # Build the final exclude query:
+        # Exclude crontab tasks that are not in our include list
+        exclude_query = Q(crontab__isnull=False) & Q(
+            crontab__id__in=excluded_hour_task_ids
+        )
+
+        return exclude_query
+
+    def _get_unique_timezone_names(self):
+        """Get a list of all unique timezone names used in CrontabSchedule"""
+        return CrontabSchedule.objects.values_list(
+            'timezone', flat=True
+        ).distinct()
+
+    def _get_timezone_offset(self, timezone_name):
+        """
+        Args:
+            timezone_name: The name of the timezone or a ZoneInfo object
+
+        Returns:
+            int: The hour offset
+        """
+        # Get server timezone
+        server_time = aware_now()
+        # Use server_time.tzinfo directly if it is already a ZoneInfo instance
+        if isinstance(server_time.tzinfo, ZoneInfo):
+            server_tz = server_time.tzinfo
+        else:
+            server_tz = ZoneInfo(str(server_time.tzinfo))
+
+        if isinstance(timezone_name, ZoneInfo):
+            timezone_name = timezone_name.key
+
+        target_tz = ZoneInfo(timezone_name)
+
+        # Use a fixed point in time for the calculation to avoid DST issues
+        fixed_dt = datetime.datetime(2023, 1, 1, 12, 0, 0)
+
+        # Calculate the offset
+        dt1 = fixed_dt.replace(tzinfo=server_tz)
+        dt2 = fixed_dt.replace(tzinfo=target_tz)
+
+        # Calculate hour difference
+        offset_seconds = (
+            dt1.utcoffset().total_seconds() - dt2.utcoffset().total_seconds()
+        )
+        offset_hours = int(offset_seconds / 3600)
+
+        return offset_hours
+
     def schedule_changed(self):
         try:
             close_old_connections()
@@ -350,13 +500,31 @@ class DatabaseScheduler(Scheduler):
     @property
     def schedule(self):
         initial = update = False
+        current_time = datetime.datetime.now()
+
         if self._initial_read:
             debug('DatabaseScheduler: initial read')
             initial = update = True
             self._initial_read = False
+            self._last_full_sync = current_time
         elif self.schedule_changed():
             info('DatabaseScheduler: Schedule changed.')
             update = True
+            self._last_full_sync = current_time
+
+        # Force update the schedule if it's been more than 5 minutes
+        if not update:
+            time_since_last_sync = (
+                current_time - self._last_full_sync
+            ).total_seconds()
+            if (
+                time_since_last_sync >= SCHEDULE_SYNC_MAX_INTERVAL
+            ):
+                debug(
+                    'DatabaseScheduler: Forcing full sync after 5 minutes'
+                )
+                update = True
+                self._last_full_sync = current_time
 
         if update:
             self.sync()
