@@ -11,7 +11,7 @@ from django_celery_beat.models import (DAYS, ClockedSchedule, CrontabSchedule,
                                        SolarSchedule)
 
 
-@pytest.mark.django_db()
+@pytest.mark.django_db
 class ActionsTests(TestCase):
 
     @classmethod
@@ -76,7 +76,7 @@ class ActionsTests(TestCase):
         self.assertTrue(e3)
 
 
-@pytest.mark.django_db()
+@pytest.mark.django_db
 class ValidateUniqueTests(TestCase):
 
     def test_validate_unique_raises_if_schedule_not_set(self):
@@ -115,7 +115,7 @@ class ValidateUniqueTests(TestCase):
         PeriodicTask(clocked=ClockedSchedule(), one_off=True).validate_unique()
 
 
-@pytest.mark.django_db()
+@pytest.mark.django_db
 class DisableTasksTest(TestCase):
 
     @classmethod
@@ -124,8 +124,7 @@ class DisableTasksTest(TestCase):
         cls.interval_schedule = IntervalSchedule.objects.create(every=10,
                                                                 period=DAYS)
 
-    @mock.patch('django_celery_beat.admin.PeriodicTaskAdmin.'
-                '_message_user_about_update')
+    @mock.patch('django_celery_beat.admin.PeriodicTaskAdmin.message_user')
     def test_disable_tasks(self, mock_message_user):
         PeriodicTask.objects.create(name='name1', task='task1', enabled=True,
                                     interval=self.interval_schedule)
